@@ -103,8 +103,6 @@ class Container(Node):
             for v in value:
                 self.__dict__[name].append(v)
 
-
-
     def __getattribute__(self, name):
 
         """
@@ -139,6 +137,54 @@ class Container(Node):
 
         return True
 
+    def diff(self, other, my_name="<<", other_name=">>"):
+
+        if type(other) != type(self):
+            return "ERROR: Cannot compare different instance types"
+
+        diff = []
+        self._diff(other, diff=diff)
+
+        output_str = ""
+        for line in diff:
+            output_str += f"\n{line[0]}:\n"
+            output_str += f"\t{my_name}: {line[1]}\n"
+            output_str += f"\t{other_name}: {line[2]}\n"
+        
+        return output_str
+
+
+    def _diff(self, other, path="", diff=[]):
+
+        my_children = self.get_child_nodes()
+        other_children = other.get_child_nodes()
+        for my_name, my_obj in my_children.items():
+            other_obj = other_children[my_name]
+
+            if isinstance(my_obj, (Leaf, LeafList)):
+                if other_children[my_name] != my_obj:
+                    diff.append( (f"{path}/{my_name}", str(my_obj), str(other_obj)) )
+
+            elif isinstance(my_obj, List):
+                for item, my_child in my_obj.items():
+                    if item not in other_obj:
+                        diff.append( (f"{path}/{my_name}", item, None) )
+                    else:
+                        new_path = f"{path}/{item}"
+                        my_child._diff(other_obj[item], new_path, diff)
+                
+                for item, other_child in other_obj.items():
+                    if item not in my_obj:
+                        diff.append( (f"{path}/{my_name}", None, item) )
+            else:
+                new_path = f"{path}/{my_name}"
+                my_obj._diff(other_obj, new_path, diff)
+
+        
+
+    def __eq__(self, other):
+        return bool(not(self.diff(other)))
+
     def _gen_xml(self, tree: etree.Element, path_filter=None):
         """
         Adds itself as an element to an existing etree object
@@ -181,7 +227,7 @@ class Leaf(Node):
     @value.setter
     def value(self, value):
         if type(value) != self._type:
-            raise TypeError(f"Invalid type for {value}, must be {self._type}")
+            raise TypeError(f"Invalid type {type(value)} for {value}, must be {self._type}")
 
         self._value = value
 
@@ -204,6 +250,11 @@ class Leaf(Node):
             else:
                 xml_node.text = str(self._value)
 
+    def __eq__(self, other):
+        return (self._value == other._value)
+
+    def __str__(self):
+        return str(self._value)
 
 class LeafList(Node):
     """
@@ -248,6 +299,11 @@ class LeafList(Node):
                 xml_node = etree.SubElement(tree, self._name, nsmap=self.nsmap)
                 xml_node.text = str(n)
 
+    def __eq__(self, other):
+        return self._values == other._values
+
+    def __str__(self):
+        return str(self._values)
 
 class List(Node):
     """
@@ -317,6 +373,11 @@ class List(Node):
     def __iter__(self):
         return iter(self._values)
 
+    def __eq__(self, other):
+        return self._values == other._values and self._keyattr == other._keyattr
+    
+    def __str__(self):
+        return str(self._values)
 
 class Choice(Container):
     """
